/***************************************************************************
 *   Copyright (C) 2007 by Gustavo Pichorim Boiko                          *
 *   boiko@mandriva.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "actionitem.h"
#include "actionpanel.h"
#include <klocale.h>
#include <kapplication.h>
#include <dcopclient.h>
#include <dcopref.h>

ActionItem::ActionItem(Panel *pan, int action)
 : PanelItem(pan)
{
    setBaseIconSize(32);
    m_action = action;
    
    if (m_action == ACTION_LOCK)
    {
        setText(i18n("Lock"));
        setIcon("lock");
    }
    else if (m_action == ACTION_LOGOUT)
    {
        setText(i18n("Logout"));
        setIcon("exit");
    }
    else if (m_action == ACTION_SWITCH_USER)
    {
        setText(i18n("Switch User"));
        setIcon("switchuser");
    }
        
}


ActionItem::~ActionItem()
{
}

bool ActionItem::launch(const QStringList &args)
{
    Q_UNUSED(args);
    if (m_action == ACTION_LOCK)
    {
        // Copied from kicker lockout applet
        QCString appname( "kdesktop" );
        int kicker_screen_number = qt_xscreen();
        if ( kicker_screen_number )
            appname.sprintf("kdesktop-screen-%d", kicker_screen_number);
        kapp->dcopClient()->send(appname, "KScreensaverIface", "lock()", "");
        return true;
    }
    else if (m_action == ACTION_LOGOUT)
    {
        DCOPRef ref("kicker", "ShutdownApplet");
        DCOPReply reply = ref.call("shutdown()");
        if (!reply.isValid())
            kapp->requestShutDown();
        return true;
    }
    else if (m_action == ACTION_SWITCH_USER)
    {
        return true;
    }
    return false;
}

bool ActionItem::canRemove()
{
    return true;
}

void ActionItem::remove()
{
    ActionPanel *pan = dynamic_cast<ActionPanel*>(panel());
    
    if (!pan)
        return;
        
    pan->removeAction(m_action);
    PanelItem::remove();
}


#include "actionitem.moc"
