/***************************************************************************
 *   Copyright (C) 2007 by Gustavo Pichorim Boiko                          *
 *   boiko@mandriva.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "actionpanel.h"
#include "actionitem.h"

#include <kconfig.h>
#include <kdebug.h>

ActionPanel::ActionPanel(int id, bool argb, PanelManager *parent, const char *name)
 : Panel(id, argb, parent, name)
{
    m_actions = 0;
}

ActionPanel::~ActionPanel()
{
}

void ActionPanel::readCfg(KConfig *cfg)
{
    /* remove all items */
    clear(false);
    
    QStringList actions = QStringList::split(",", "Logout,Lock,SwitchUser");
    actions = cfg->readListEntry("Actions", actions);
    
    m_actions = 0;
    
    for (unsigned int i = 0; i < actions.count(); ++i)
    {
        QString action = actions[i];
        
        if (action == "Logout")
            m_actions |= ACTION_LOGOUT;
        else if (action == "Lock")
            m_actions |= ACTION_LOCK;
        else if (action == "SwitchUser")
            m_actions |= ACTION_SWITCH_USER;
    }
    
    /* FIXME */
    if (m_actions & ACTION_LOCK)
        addItem(new ActionItem(this, ACTION_LOCK), false);
    if (m_actions & ACTION_LOGOUT)
        addItem(new ActionItem(this, ACTION_LOGOUT), false);
    if (m_actions & ACTION_SWITCH_USER)
        addItem(new ActionItem(this, ACTION_SWITCH_USER), false);
    updateSize();
}

void ActionPanel::removeAction(int action)
{
    m_actions &= ~action;

    /* save the remaining actions to the config file */
    QStringList actions;
    if (m_actions & ACTION_LOCK)
        actions.append("Lock");
    if (m_actions & ACTION_LOGOUT)
        actions.append("Logout");
    if (m_actions & ACTION_SWITCH_USER)
        actions.append("SwitchUser");

    writeConfigEntry("Actions", actions);
}

#include "actionpanel.moc"
