/***************************************************************************
 *   Copyright (C) 2007 by Gustavo Pichorim Boiko                          *
 *   boiko@mandriva.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "diritem.h"
#include "dirpanel.h"
#include "panel.h"
#include <kfileitem.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <ktrader.h>
#include <krun.h>
#include <kdebug.h>
#include <kio/netaccess.h>
#include <kurldrag.h>
#include <kurl.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <kdesktopfile.h>
#include <kio/netaccess.h>

DirItem::DirItem(Panel *pan, KFileItem *itm)
 : PanelItem(pan)
{
    m_item = itm;
    
    QString name = itm->name();
    if (name.endsWith(".desktop"))
    {
        KDesktopFile desktop(KIO::NetAccess::mostLocalURL(itm->url(), 0).path());
        QString tmp = desktop.readName();
        if (!tmp.isEmpty())
            name = tmp;
    }
    setText(name);
    setIcon(itm->iconName());
}


DirItem::~DirItem()
{
}

bool DirItem::launch(const QStringList &args)
{
    Q_UNUSED(args);
    m_item->run();
    return true;
}

void DirItem::slotRefresh()
{
    setText(m_item->name());
    setIcon(m_item->iconName());
    
    refreshPanel();
}

bool DirItem::canHandle(QDropEvent *event)
{
    if (!KURLDrag::canDecode(event))
        return false;
    if (!m_item->isDir() || !m_item->isWritable())
        return false;
    return true;
}

bool DirItem::handleDrop(QDropEvent *event)
{
    if (!KURLDrag::canDecode(event))
        return false;
    if (!m_item->isDir() || !m_item->isWritable())
        return false;
        
    KURL::List files;
    KURLDrag::decode(event, files);
    KURL url;
    QString location = m_item->url().url();
    bool status = true;
    for (unsigned int i = 0; i < files.count(); ++i)
    {
        url = files[i];
        kdDebug() << location + "/" + url.fileName() << endl;
        if (!KIO::NetAccess::copy(url, location + "/" + url.fileName(), panel()))
        {
            kdDebug() << "Could not copy file " << url << endl;
            status = false;
        }
    }
    return status;
}

KFileItem * DirItem::item() const
{
    return m_item;
}

bool DirItem::canRemove()
{
    return m_item->isWritable();
}

void DirItem::remove()
{
    if (KMessageBox::warningYesNo(panel(), i18n("Do you really want to remove the file %1?"
                                  "Note that the file will be removed from the disk.").arg(text()),
                                  "KPanels - Remove item") == KMessageBox::Yes)
    {
        KIO::NetAccess::del(m_item->url(), panel());
    }
}


#include "diritem.moc"
