/***************************************************************************
 *   Copyright (C) 2007 by Gustavo Pichorim Boiko                          *
 *   boiko@mandriva.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "dirpanel.h"
#include "diritem.h"
#include <kurldrag.h>
#include <kio/netaccess.h>
#include <kdebug.h>
#include <klocale.h>
#include <qinputdialog.h>
#include <kpopupmenu.h>
#include <kconfigbackend.h>

DirPanel::DirPanel(int id, bool argb, PanelManager *parent, const char *name)
 : Panel(id, argb, parent, name)
{
    m_dirLister.setAutoUpdate(true);
    connect(&m_dirLister, SIGNAL(newItems(const KFileItemList&)),
            this, SLOT(slotNewItems(const KFileItemList&)));
            
    connect(&m_dirLister, SIGNAL(deleteItem( KFileItem*)),
            this, SLOT(slotDeleteItem(KFileItem*)));
            
}


DirPanel::~DirPanel()
{
}

void DirPanel::slotNewItems(const KFileItemList &items)
{
    for (KFileItemListIterator it(items); *it; ++it)
    {
        DirItem *item = new DirItem(this, *it);
        connect(&m_dirLister, SIGNAL(refreshItems(const KFileItemList&)),
                item, SLOT(slotRefresh()));
        addItem(item, false);
    }
    updateSize();
}

void DirPanel::readCfg(KConfig *cfg)
{
    clear(false);
    m_url = cfg->readPathEntry("URL", "system:/");
    m_dirLister.openURL(m_url);
}

bool DirPanel::canHandle(QDropEvent *event)
{
    if (!KURLDrag::canDecode(event))
        return false;
    
    KFileItem *root = m_dirLister.rootItem();
    if (!root->isDir() || !root->isWritable())
        return false;
    
    return true;
}

bool DirPanel::handleDrop(QDropEvent *event)
{
    if (!KURLDrag::canDecode(event))
        return false;
    
    KFileItem *root = m_dirLister.rootItem();
    if (!root->isDir() || !root->isWritable())
        return false;
        
    KURL::List files;
    
    KURLDrag::decode(event, files);
    KURL url;
    QString location = root->url().url();
    
    bool status = true;
    for (unsigned int i = 0; i < files.count(); ++i)
    {
        url = files[i];
        kdDebug() << location + "/" + url.fileName() << endl;
        if (!KIO::NetAccess::copy(url, location + "/" + url.fileName(), this))
        {
            kdDebug() << "Could not copy file " << url << endl;
            status = false;
        }
    }
    m_dirLister.updateDirectory(root->url());
    return status;
}

void DirPanel::slotDeleteItem(KFileItem *item)
{
    ItemList list = items();
    ItemList::iterator it = list.begin();
    
    for (; it != list.end(); ++it)
    {
        DirItem *dirItem = dynamic_cast<DirItem*>(*it);
        if (!dirItem)
            continue;
            
        if (dirItem->item() == item)
        {
            removeItem(dirItem);
            break;
        }
    }
}

void DirPanel::panelMenu(KPopupMenu *menu)
{
    menu->insertItem(loadIcon("fileopen"), i18n("Change URL"), 
                     this, SLOT(slotChangeURL()));
}

void DirPanel::slotChangeURL()
{
    bool ok;
    QString message = i18n("Change URL:");
    QString url = QInputDialog::getText("KPanels", message, QLineEdit::Normal,
                                         m_url.url(), &ok, this);
    if (ok && !url.isEmpty())
    {
        m_url = url;
        writeConfigEntry("URL", m_url.url());
        clear(false);
        m_dirLister.openURL(m_url);
    }
}

#include "dirpanel.moc"
