/***************************************************************************
 *   Copyright (C) 2007 by Gustavo Pichorim Boiko   *
 *   boiko@mandriva.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef DIRPANEL_H
#define DIRPANEL_H

#include "panel.h"
#include <kurl.h>
#include <kdirlister.h>

/**
 * A panel that uses a directory as its item list. The directory can be any
 * kio-enabled path (including media:/ settings:/ and so)
 */
class DirPanel : public Panel
{
Q_OBJECT
public:
    DirPanel(int id, bool argb, PanelManager *parent = 0, const char *name = 0);

    ~DirPanel();

    virtual QString type() const { return "Dir"; }
    
    /* config stuff */
    virtual void readCfg(KConfig *cfg);
    
    /* drag and drop stuff */
    virtual bool canHandle(QDropEvent *event);
    virtual bool handleDrop(QDropEvent *event);
    
protected:
    /**
     * Reimplemented from \ref Panel
     * @see \ref Panel::panelMenu()
     */
    virtual void panelMenu(KPopupMenu *menu);
    
private:
    KURL m_url;
    KDirLister m_dirLister;
    
    void listEntries();
    
private slots:
    void slotNewItems(const KFileItemList &items);
    void slotDeleteItem(KFileItem *item);
    void slotChangeURL();

};

#endif
