/***************************************************************************
 *   Copyright (C) 2007 by Gustavo Pichorim Boiko                          *
 *   boiko@mandriva.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "panelmanager.h"
#include "dockwidget.h"
#include <kuniqueapplication.h>
#include <kaboutdata.h>
#include <kcmdlineargs.h>
#include <klocale.h>
#include <kconfig.h>

#include <stdlib.h>

#if defined(Q_WS_X11) && QT_VERSION >= 0x030300
# define COMPOSITE
#endif

// disable composite until I discover why ksmserver is starting two instances of kpanels when using KApplication

#undef COMPOSITE

#include <X11/Xlib.h>
#ifdef COMPOSITE
# include <X11/extensions/Xrender.h>
#endif

static const char description[] =
    I18N_NOOP("Desktop application launcher");

static const char version[] = "0.99.0";

static KCmdLineOptions options[] =
{
//    { "+[URL]", I18N_NOOP( "Document to open" ), 0 },
    KCmdLineLastOption
};

int main(int argc, char **argv)
{
    Display *dpy = XOpenDisplay(NULL);
    if (!dpy)
    {
        fprintf(stderr, "ERROR: couldn't open display %s\n", XDisplayName(NULL));
        exit(1);
    }
    
    
    KAboutData about("kpanels", I18N_NOOP("KPanels"), version, description,
		     KAboutData::License_GPL, "(C) 2007 Gustavo Pichorim Boiko", 0, 0, "boiko@mandriva.com");
    about.addAuthor( "Gustavo Pichorim Boiko", 0, "boiko@mandriva.com" );
    KCmdLineArgs::init(argc, argv, &about);
    KCmdLineArgs::addCmdLineOptions( options );
    KUniqueApplication *app;
    PanelManager *manager = 0;
    DockWidget *dock = 0;
    bool use_argb = false;
    
#ifdef COMPOSITE
    int screen = DefaultScreen( dpy );
    Colormap colormap = 0;
    Visual *visual = 0;
    int event_base, error_base;
    
    if (XRenderQueryExtension(dpy, &event_base, &error_base))
    {
        int nvi;
        XVisualInfo templ;
        templ.screen  = screen;
        templ.depth   = 32;
        templ.c_class = TrueColor;
        XVisualInfo *xvi = XGetVisualInfo( dpy, VisualScreenMask | VisualDepthMask
                           | VisualClassMask, &templ, &nvi );
        if (xvi)
        {
            for (int i = 0; i < nvi; i++)
            {
                XRenderPictFormat *format = XRenderFindVisualFormat(dpy, xvi[i].visual);
                if (format->type == PictTypeDirect && format->direct.alphaMask)
                {
                    visual = xvi[i].visual;
                    colormap = XCreateColormap(dpy, RootWindow( dpy, screen ), visual,
                                               AllocNone);
                    use_argb = true;
                    break;
                }
            }
        }
    }
    use_argb = false;
    if (use_argb)
        app = new KApplication(dpy, Qt::HANDLE(visual), Qt::HANDLE(colormap));
    else
        app = new KApplication(dpy);
#else
    app = new KUniqueApplication(dpy);
#endif

    // no session.. just start up normally
    KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
    
    /// @todo do something with the command line args here

    manager = new PanelManager(use_argb);
    
    dock = new DockWidget();
    QObject::connect (dock, SIGNAL(quitSelected()), app, SLOT(quit()));
    dock->show();
    
    args->clear();

    return app->exec();
}
