/***************************************************************************
 *   Copyright (C) 2007 by Gustavo Pichorim Boiko                          *
 *   boiko@mandriva.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "panel.h"
#include "panelitem.h"
#include "panelmanager.h"

#include <kconfig.h>
#include <krootpixmap.h>
#include <kiconeffect.h>
#include <netwm.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qinputdialog.h>
#include <kdebug.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <kpopupmenu.h>
#include <klocale.h>
#include <kapplication.h>
#include <kcolordialog.h>
#include <kdirwatch.h>

// config stuff
#include "panelconfig.h"
#include "configpagegeneral.h"

Panel::Panel(int id, bool argb, PanelManager *parent, const char *name)
 : QWidget(0, name, Qt::WGroupLeader | WStyle_Customize | WStyle_NoBorder |
                        WDestructiveClose ),
   m_selection(this)
{
    can_delete = false;
    m_parent = parent;
    m_useArgb = argb;
    m_id = id;
    m_bg = QPixmap(size());
    m_color = Qt::gray;
    m_selection.setColor(m_color);
    
    m_titleFont.setPointSize(23);
    m_itemFont.setPointSize(14);
    m_locked = false;
    
    // Set window properties
    NETWinInfo *info = new NETWinInfo(qt_xdisplay(), winId(), qt_xrootwin(),
                                      NET::WMState );
    info->setDesktop(NETWinInfo::OnAllDesktops);
    info->setState(NETWinInfo::SkipTaskbar | NETWinInfo::SkipPager |
                        NETWinInfo::Hidden | NETWinInfo::KeepBelow,
                   NETWinInfo::SkipTaskbar | NETWinInfo::SkipPager |
                        NETWinInfo::Hidden | NETWinInfo::KeepBelow);
    
    /**
     * If we are using ARGB visuals, we don't need to copy the background's_
     * pixmap, we just need to draw using different alpha values. But when_
     * there is no ARGB visual available, we have to use KRootPixmap to copy
     * the background contents before bitBlitting our interface.
     */
    if (!m_useArgb)
    {
        m_rootPixmap = new KRootPixmap(this);
        m_rootPixmap->setCustomPainting( true );
        
        //signals_
        connect(m_rootPixmap, SIGNAL(backgroundUpdated (const QPixmap &)),
                this, SLOT(slotUpdateBackground( const QPixmap& )));
        m_rootPixmap->start();
    }
    else
    {   
        QPixmap pix(size());
        pix.fill( QColor(Qt::black.rgb(), 0x00000000));
        
        slotUpdateBackground(pix);
    }
    
    updateSize();
    setAcceptDrops(TRUE);
    setMouseTracking(TRUE);
    m_moving = false;
    
}


Panel::~Panel()
{
   
}

void Panel::slotUpdateBackground(const QPixmap &pix)
{
    m_bg = pix;
    renderBackground();
    repaint();
}

void Panel::renderBackground()
{
    int top = 0;
    QRect r = toBase(QRect(0, top, width(), height()));
    QPainter p(&m_bg);
    //p.scale(m_scaleX, m_scaleY);
    //p.setWorldMatrix(m_matrix);
    if (!m_top.isNull())
    {
        r.setHeight(m_top.height());
        p.drawPixmap(fromBase(r), m_top);
        top += fromBase(r).height();
        
        r.setTop(r.top()+10);
        // draw the title with a shadow
        QRect r2 = QRect(r.left()+2, r.top()+2, r.width(), r.height());
        QFont f = m_titleFont;
        f.setPointSize(fromBase(QSize(f.pointSize(), f.pointSize())).width());
        p.setFont(f);
        if (m_useShadow)
        {
            p.setPen(Qt::black);
            p.drawText(fromBase(r2), Qt::AlignCenter, m_title);
        }
        p.setPen(m_textColor);
        p.drawText(fromBase(r), Qt::AlignCenter, m_title);
        r.moveTop(r.top() + r.height());
    }
    if (!m_center.isNull())
    {
        r.setHeight(m_center.height());
        
         //draw the base background for items
        QRect r2 = fromBase(r);
        r2.moveTop(top);
        QImage img = m_center.convertToImage();
        img = img.smoothScale(fromBase(r).size());
        r2.setHeight(fromBase(QSize(1,m_center.height() * m_itemList.count())).height());
        
        p.drawTiledPixmap(r2, QPixmap(img));
        r.setHeight(m_center.height());
        
        for (unsigned int i=0; i < m_itemList.count(); ++i)
        {
            QRect itemRect(QRect(r.left()+20,r.top()+5, r.width()-60, r.height()-10));
            r.moveTop(r.top() + r.height());
            m_itemList[i]->paint(&p, fromBase(itemRect));
        }
        top += r2.height();
    }
    if (!m_bottom.isNull())
    {
        r.setHeight(m_bottom.height());
        QRect r2 = fromBase(r);
        r2.moveTop(top);
        p.drawPixmap(r2, m_bottom);
    }
    p.flush();
}

void Panel::setColor(const QColor &color)
{
    if (m_color == color)
        return;
        
    m_color = color;
    m_selection.setColor(color);
    if (m_parent)
    {
        m_parent->applyTheme(this);
    }
    else
    {
        if (!m_top.isNull())
            applyColor(m_top, true);
        if (!m_bottom.isNull())
            applyColor(m_bottom, true);
        if (!m_center.isNull())
            applyColor(m_center, true);
        updateSize();
    }
}

void Panel::applyColor(QPixmap &pix, bool reset)
{
    if (!m_color.isValid())
        return;
        
    KIconEffect fx;
    if (reset)
        pix = fx.apply(pix, KIconEffect::ToGray, 1, Qt::gray, 0);
    pix = fx.apply(pix, KIconEffect::Colorize, 1, m_color, 0);
}

void Panel::updateSize()
{
    int h = 0;
    if (!m_top.isNull())
        h += m_top.height();
    if (!m_bottom.isNull())
        h += m_bottom.height();
    if (!m_center.isNull())
        h += m_center.height() * m_itemList.count();
    QSize s = toBase(size());
    s.setHeight(h);
    resize(fromBase(s));
    refresh();
}

void Panel::setTop(const QPixmap & top)
{
    m_top = top;
    applyColor(m_top);
    updateSize();
}

void Panel::setBottom(const QPixmap & bottom)
{
    m_bottom = bottom;
    applyColor(m_bottom);
    updateSize();
}

void Panel::setCenter(const QPixmap & center)
{
    m_center = center;
    applyColor(m_center);
    updateSize();
}

void Panel::paintEvent(QPaintEvent *event)
{
    QPixmap pix = m_bg;
    
    if (!event->region().intersect(m_selection.rect()).isNull())
    {
        /* do NOT remove the detach() call here: QPixmap is implicitly shared
         * and not detaching the copy will end up damaging the rendered 
         * background
         */
        pix.detach();
        m_selection.paint(pix);
        XFlush(pix.x11Display());
    }
    
    QMemArray<QRect> rects = event->region().rects();
    for (unsigned int i=0; i < rects.size(); ++i)
        bitBlt(this, rects[i].topLeft(), &pix, rects[i], Qt::CopyROP, false);
}

void Panel::addItem(PanelItem *item, bool update)
{
    m_itemList.append(item);
    if (update)
        updateSize();
}

void Panel::refresh()
{
    if (m_useArgb)
    {
        QPixmap pix(size());
        pix.fill( QColor(Qt::black.rgb(), 0x00000000));
        slotUpdateBackground(pix);
    }
    else
    {
        m_rootPixmap->repaint(true);
    }
}

void Panel::mousePressEvent(QMouseEvent *e)
{
    if (e->button() == QMouseEvent::LeftButton)
    {
        m_pressedItem = itemAt(e->pos());
        
        if (!m_pressedItem && !m_locked)
        {
            m_moving = true;
            m_clickPos = e->pos();
        }
    }
}

void Panel::mouseReleaseEvent(QMouseEvent *e)
{
    if (m_moving)
    {
        m_moving = false;
        emit positionChanged(this);
    }
    else
    {
        PanelItem *item = itemAt(e->pos());
    
        if (e->button() == QMouseEvent::LeftButton)
        {
            if (!item)
                return; /* FIXME */
            if (item == m_pressedItem)
                item->launch();
        }
    }
}

void Panel::mouseDoubleClickEvent(QMouseEvent *e)
{
    Q_UNUSED(e);
}

void Panel::mouseMoveEvent(QMouseEvent *e)
{
    if (m_moving)
    {
        move(e->globalPos() - m_clickPos);
        return;
    }
    
    PanelItem *item = itemAt(e->pos());
    if (!item)
    {
        m_selection.unselect();
        return;
    }
    
    m_selection.selectItem(item);
}

PanelItem *Panel::itemAt(QPoint pos)
{
    pos = toBase(pos);
    PanelItem *item;
    for (unsigned int i = 0; i < m_itemList.count(); ++i)
    {
        item = m_itemList[i];
        if (item->pointIsOver(pos))
            return item;
    
    }
    return NULL;
}

void Panel::dragEnterEvent(QDragEnterEvent *event)
{
    PanelItem *item = itemAt(event->pos());
    
    if (!item)
    {
        // only accept drops if the panel is 'writable'
        if(canHandle(event))
            event->accept();
    }
    else
    {
        if (item->canHandle(event))
            event->accept();
    }
}

void Panel::dragMoveEvent(QDragMoveEvent *event)
{
    PanelItem *item = itemAt(event->pos());
    
    if (!item)
    {
        // only accept drops if the panel is 'writable'
        if(canHandle(event))
            event->accept();
        else
            event->ignore();
    }
    else
    {
        if (item->canHandle(event))
            event->accept();
        else
            event->ignore();
    }
}

void Panel::dragLeaveEvent(QDragLeaveEvent *event)
{
    Q_UNUSED(event);
}

void Panel::dropEvent(QDropEvent *event)
{
    PanelItem *item = itemAt(event->pos());
    
    if (!item)
    {
        if (handleDrop(event))
            event->accept();
        else
            event->ignore();
    }
    else
    {
        if (item->handleDrop(event))
            event->accept();
        else
            event->ignore();
    }
}

bool Panel::canHandle(QDropEvent *event)
{
    Q_UNUSED(event);
    // the virtual panel cannot handle any drag-and-drop
    return false;
}



bool Panel::handleDrop(QDropEvent * event)
{
    Q_UNUSED(event);
    
    return false;
}

void Panel::readConfig()
{
    KConfig cfg("kpanelsrc");
    cfg.setGroup("Panel_" + QString::number(m_id));
    
    
    QSize s = toBase(size());
    s.setWidth(cfg.readNumEntry("Width", 300));
    resize(fromBase(s));
    
    m_title = cfg.readEntry("Title", i18n("Panel"));
    setColor(cfg.readColorEntry("Color", &Qt::gray));
    m_locked = cfg.readBoolEntry("Locked", false);
    m_titleFont = cfg.readFontEntry("TitleFont", &m_titleFont);
    m_itemFont = cfg.readFontEntry("ItemFont", &m_itemFont);
    m_textColor = cfg.readColorEntry("TextColor", &Qt::white);
    m_useShadow = cfg.readBoolEntry("UseShadow", true);
    
    readCfg(&cfg);
    updateSize();
}

void Panel::readCfg(KConfig *cfg)
{
    // this is for the reimplemented classes
    Q_UNUSED(cfg);
}

int Panel::id()
{
    return m_id;
}

void Panel::removeItem(PanelItem *item, bool update)
{
    ItemList::iterator it = m_itemList.find(item);
    
    if (it != m_itemList.end())
        m_itemList.erase(it);
    m_selection.unselect();
    
    delete item;
    if (update)
        updateSize();
}

ItemList Panel::items() const
{
    return m_itemList;
}

void Panel::clear(bool update)
{
    m_selection.unselect();
    ItemList::iterator it = m_itemList.begin();
    ItemList::iterator end = m_itemList.end();
    for (; it != end; ++it)
        delete (*it);
    
    m_itemList.clear();
    
    if (update)
        updateSize();
}

void Panel::contextMenuEvent(QContextMenuEvent *event)
{
    PanelItem *item = itemAt(event->pos());
    KPopupMenu menu(this);
    int id;
    
    menu.insertTitle(m_title);
    menu.setCheckable(TRUE);
    
    if (!item)
    {
        panelMenu(&menu);
        
        if (menu.count())
            menu.insertSeparator();
        
        id = menu.insertItem(loadIcon("edit"), i18n("Change Panel Title"),
                             this, SLOT(slotChangeTitle()));
        id = menu.insertItem(loadIcon("colorize"), i18n("Change Panel Color"),
                             this, SLOT(slotChangeColor()));
        /* Lock the panel not to move it when dragging */
        id = menu.insertItem(loadIcon("lock"), i18n("Lock Panel"),
                             this, SLOT(toggleLocked()));
        menu.setItemChecked(id, m_locked);
    
        id = menu.insertItem(loadIcon("configure"), i18n("Configure Panel"),
                             this, SLOT(slotConfigure()));
        menu.insertSeparator();
    
        menu.insertItem(loadIcon("fileclose"), i18n("Close KPanels"), 
                        kapp, SLOT(quit()));
    }
    else
    {
        item->itemMenu(&menu);
        if (menu.count())
            menu.insertSeparator();
        
        if (item->canRemove())
            menu.insertItem(loadIcon("editdelete"), i18n("Remove Item"),
                            item, SLOT(slotRemove()));
    }
    if (menu.count())
        menu.exec(event->globalPos());
}

void Panel::setTitle(const QString &title)
{
    m_title = title;
    refresh();
}

QString Panel::title() const
{
    return m_title;
}

void Panel::toggleLocked()
{
    m_locked = !m_locked;
    writeConfigEntry("Locked", m_locked);
}

void Panel::setLocked(bool l)
{
    m_locked = l;
}

bool Panel::locked() const
{
    return m_locked;
}

QPixmap Panel::loadIcon(const QString &icon)
{
    return KGlobal::iconLoader()->loadIcon(icon, KIcon::Small);
}

void Panel::panelMenu(KPopupMenu *menu)
{
    Q_UNUSED(menu);
}

void Panel::setScale(double scaleX, double scaleY)
{
    /* calculate the proportional size */
    QSize s = toBase(size());
    
    m_matrix.reset();
    m_matrix.scale(scaleX, scaleY);
    m_invMatrix = m_matrix.invert();
    
    resize(fromBase(s)); 
    updateSize();
}

void Panel::slotChangeColor()
{
    QColor color;
    
    if (KColorDialog::getColor(color, m_color, this))
    {
        if (color == m_color)
            return;
        
        setColor(color);
        writeConfigEntry("Color", color);
    }
}



QPoint Panel::fromBase(const QPoint &p)
{
    return m_matrix.map(p);
}

QPoint Panel::toBase(const QPoint &p)
{
    return m_invMatrix.map(p);
}

QSize Panel::fromBase(const QSize &s)
{
    return m_matrix.mapRect(QRect(QPoint(0,0), s)).size();
}

QSize Panel::toBase(const QSize &s)
{
    return m_invMatrix.mapRect(QRect(QPoint(0,0), s)).size();
}

QRect Panel::fromBase(const QRect &r)
{
    return m_matrix.mapRect(r);
}

QRect Panel::toBase(const QRect &r)
{
    return m_invMatrix.mapRect(r);
}

QColor Panel::color() const
{
    return m_color;
}

void Panel::writeConfigEntry(const QString &entry, const QVariant &value)
{
    KDirWatch::self()->stopScan();
    
    KConfig cfg("kpanelsrc");
    cfg.setGroup("Panel_" + QString::number(m_id));
    cfg.writeEntry(entry, value);
    cfg.sync();
    
    KDirWatch::self()->startScan(false, false);
}

void Panel::slotChangeTitle()
{
    bool ok;
    QString message = i18n("Title:");
    QString text = QInputDialog::getText("KPanels", message, QLineEdit::Normal,
                                         m_title, &ok, this);
    if (ok)
    {
        setTitle(text);
        writeConfigEntry("Title", text);
    }
}

void Panel::writeConfig()
{
    KConfig cfg("kpanelsrc");
    cfg.setGroup("Panel_" + QString::number(m_id));
    
    
    QSize s = toBase(size());
    cfg.writeEntry("Width", s.width());
    cfg.readEntry("Title", m_title);
    cfg.writeEntry("Color", m_color);
    cfg.writeEntry("Locked", m_locked);
    cfg.writeEntry("TitleFont", m_titleFont);
    cfg.writeEntry("ItemFont", m_itemFont);
    cfg.writeEntry("UseShadow", m_useShadow);
    cfg.writeEntry("TextColor", m_textColor);
}

void Panel::slotConfigure()
{
    PanelConfig config(this);
    ConfigPageGeneral *page = new ConfigPageGeneral(this);
    config.addPage(page);
    config.exec();
}

void Panel::setTitleFont(const QFont &font)
{
    m_titleFont = font;
    updateSize();
}

QFont Panel::titleFont() const
{
    return m_titleFont;
}

void Panel::setItemFont(const QFont &font)
{
    m_itemFont = font;
    updateSize();
}

QFont Panel::itemFont() const
{
    return m_itemFont;
}

bool Panel::useShadow() const
{
    return m_useShadow;
}

void Panel::setUseShadow(bool shadow)
{
    m_useShadow = shadow;
    updateSize();
}

QColor Panel::textColor() const
{
    return m_textColor;
}

void Panel::setTextColor(const QColor &color)
{
    m_textColor = color;
    updateSize();
}


#include "panel.moc"
