/***************************************************************************
 *   Copyright (C) 2007 by Gustavo Pichorim Boiko                          *
 *   boiko@mandriva.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "panelitem.h"
#include "panel.h"

#include <qpainter.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <kdebug.h>
#include <kpopupmenu.h>
#include <klocale.h>

PanelItem::PanelItem(Panel *pan)
: QObject(pan)
{
    m_panel = pan;
    m_baseIconSize = 48; // a sane default
}


PanelItem::~PanelItem()
{
}

void PanelItem::paint(QPainter *p, QRect r)
{
    m_rect = panel()->toBase(r);
    p->save();
    int left = r.left();
    int vcenter = (int)((r.top() + r.bottom()) * 0.5);
    
    // check if the current icon size is the one we need to draw,
    // if it is not, load the icon again
    int size = panel()->fromBase(QSize(m_baseIconSize,m_baseIconSize)).width();
    if (size != m_iconSize)
    {
        m_iconSize = size;
        m_icon = loadIcon(m_iconName);
    }
    p->drawPixmap(left, vcenter - (int)(m_icon.height() * 0.5), m_icon);
    left += m_icon.width() + panel()->fromBase(QSize(20,20)).width();
    
    r.setLeft(left);
    QRect r2 = QRect(left+2, r.top()+2, r.width(), r.height());
    
    // the item font is set in the panel itself
    QFont itemFont = panel()->itemFont();
    itemFont.setPointSize(panel()->fromBase(QSize(itemFont.pointSize(), itemFont.pointSize())).width());
    p->setFont(itemFont);
    if (m_panel->useShadow())
    {
        p->setPen(Qt::black);
        p->drawText(r2, Qt::AlignVCenter | Qt::WordBreak, m_text);
    }
    
    p->setPen(m_panel->textColor());
    p->drawText(r, Qt::AlignVCenter | Qt::WordBreak, m_text);
    
    p->restore();
}

QString PanelItem::text() const
{
    return m_text;
}

QPixmap PanelItem::icon() const
{
    return m_icon;
}

void PanelItem::setText(const QString &txt)
{
    m_text = txt;
}

void PanelItem::setIcon(const QString &ico)
{
    m_iconName = ico;
    m_iconSize = panel()->fromBase(QSize(m_baseIconSize,m_baseIconSize)).width();
    m_icon = loadIcon(ico);
}

bool PanelItem::launch(const QStringList &args)
{
    Q_UNUSED(args);
    // do nothing in the default implementation
    return true;
}

Panel *PanelItem::panel()
{
    return m_panel;
}

QPixmap PanelItem::loadIcon(const QString &ico)
{
    return KGlobal::iconLoader()->loadIcon(ico, KIcon::Desktop, m_iconSize);
}

void PanelItem::refreshPanel()
{
    m_panel->refresh();
}

bool PanelItem::pointIsOver(QPoint pos)
{
    if (m_rect.contains(pos))
        return true;
    return false;
}

bool PanelItem::canHandle(QDropEvent *event)
{
    Q_UNUSED(event);
    /* the default item cannot handle drag-and-drop */
    return false;
}

bool PanelItem::handleDrop(QDropEvent *event)
{
    Q_UNUSED(event);
    
    return false;
}

bool PanelItem::canRemove()
{
    return true;
}

void PanelItem::remove()
{
    if (canRemove())
        m_panel->removeItem(this, true);
}

void PanelItem::slotRemove()
{
    remove();
}

void PanelItem::itemMenu(KPopupMenu *menu)
{
    int id;
    id = menu->insertItem(loadIcon("exec"), i18n("Launch Item"),
                             this, SLOT(launch()));
}

QRect PanelItem::rect() const
{
    return m_rect;
}

void PanelItem::setBaseIconSize(int size)
{
    m_baseIconSize = size;
}

#include "panelitem.moc"
