/***************************************************************************
 *   Copyright (C) 2007 by Gustavo Pichorim Boiko                          *
 *   boiko@mandriva.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef SELECTION_H
#define SELECTION_H

#include <qobject.h>
#include <qcolor.h>
#include <qsize.h>
#include <qtimer.h>

#define ANIM_MOVE       1
#define ANIM_FADEIN     2
#define ANIM_FADEOUT    4

class Panel;
class PanelItem;

/**
 * This is the class in charge of painting and animating selecions on the panel.
 */
class Selection : public QObject
{
Q_OBJECT
public:
    Selection(Panel *parent, const char *name = 0);
    ~Selection();
    
    void setColor(const QColor &c);
    void selectItem(PanelItem *item);
    void setSize(const QSize &size);
    void unselect();
    void paint(QPixmap &p);
    QRect rect() const;
    
private slots:
    void slotAnimate();
    
private:
    QColor m_color;
    PanelItem *m_item;
    QRect m_rect;
    Panel *m_panel;
    
    /* animation stuff */
    QTimer m_timer;
    float m_moveDelta;
    float m_fadeDelta;
    float m_topOrigin;
    int m_fadeStep;
    int m_moveStep;
    float m_alpha;
    int m_anim;
    
    QRect expand(const QRect &rect);
   
};

#endif
