/***************************************************************************
 *   Copyright (C) 2007 by Gustavo Pichorim Boiko                          *
 *   boiko@mandriva.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "serviceitem.h"
#include <kglobal.h>
#include <kiconloader.h>
#include <kdebug.h>
#include <dcopref.h>
#include <kmimetype.h>
#include <kurldrag.h>
#include <qfile.h>

ServiceItem::ServiceItem(Panel *pan, KService::Ptr svc)
 : PanelItem(pan)
{
    m_service = svc;
    
    setText(svc->name());
    setIcon(svc->icon());
}


ServiceItem::~ServiceItem()
{
}

bool ServiceItem::launch(const QStringList &args)
{
    /* FIXME: there should be a better way to do that */
    DCOPRef ref("klauncher", "klauncher");
    DCOPReply reply = ref.call("start_service_by_desktop_path", m_service->storageId(), args);
    
    /* FIXME: check the value of the reply when it is valid */
    if (!reply.isValid())
    {
        kdDebug() << "kpanels: WARNING: Could not launch service "
                  << m_service->name() << endl;
        return false;
    }
    return true;
}

bool ServiceItem::canHandle(QDropEvent *event)
{
    QStringList mimeTypes = m_service->serviceTypes();
    
    if (!KURLDrag::canDecode(event))
        return false;
    KURL::List list;
    
    KURLDrag::decode(event, list);
    KURL url;
    KMimeType::Ptr mime;
    bool canHandle = false;
    for (unsigned int i = 0; i < list.count(); ++i)
    {
        canHandle = false;
        url = list[i];
        mime = KMimeType::findByURL(url);
        for ( QStringList::Iterator it = mimeTypes.begin(); it != mimeTypes.end(); ++it )
        {
            if (mime->is(*it))
            {
                canHandle = true;
                break;
            }
        }
        if (!canHandle)
            break;
    }
    return canHandle;
}

bool ServiceItem::handleDrop(QDropEvent *event)
{
    KURL::List list;
    KURLDrag::decode(event, list);
    QStringList args = list.toStringList();
    
    return launch(args);
}

bool ServiceItem::canRemove()
{
    QFile f(m_service->storageId());
      
    if (!f.exists())
        return true;
        
    if (f.open(IO_ReadWrite))
    {
        f.close();
        return true;
    }
    return false;
}

void ServiceItem::remove()
{
    if (canRemove())
        QFile::remove(m_service->storageId());
        
    PanelItem::remove();
}


#include "serviceitem.moc"
