/***************************************************************************
 *   Copyright (C) 2007 by Gustavo Pichorim Boiko                          *
 *   boiko@mandriva.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "servicepanel.h"
#include "serviceitem.h"
#include <kglobal.h>
#include <kstandarddirs.h>
#include <kservice.h>
#include <kmimetype.h>
#include <kurldrag.h>
#include <kio/netaccess.h>
#include <kdirwatch.h>

#include <kdebug.h>

ServicePanel::ServicePanel(int id, bool argb, PanelManager *parent, const char *name)
 : Panel(id, argb, parent, name)
{
    m_name = "panel_" + QString::number(id);

    QStringList dirs = KGlobal::dirs()->findDirs("appdata", m_name);
    for (unsigned int i = 0; i < dirs.count(); ++i)
        KDirWatch::self()->addDir(dirs[i], true, false);
    
    connect(KDirWatch::self(), SIGNAL(dirty(const QString&)),
            this, SLOT(listServices()));
    connect(KDirWatch::self(), SIGNAL(created(const QString&)),
            this, SLOT(listServices()));
    connect(KDirWatch::self(), SIGNAL(deleted(const QString&)),
            this, SLOT(listServices()));
            
    listServices();
}


ServicePanel::~ServicePanel()
{
}

void ServicePanel::listServices()
{
    clear(false);
    QStringList items;
    items = KGlobal::dirs()->findAllResources("appdata", m_name + "/*.desktop", false, true);
    
    
    for ( QStringList::Iterator it = items.begin(); it != items.end(); ++it ) 
    {
        KService::Ptr svc = KService::serviceByStorageId(*it);
        
        if (!svc)
            continue;
            
       addItem(new ServiceItem(this, svc), false);
    }
    updateSize();
}

bool ServicePanel::canHandle(QDropEvent *event)
{
    if (!KURLDrag::canDecode(event))
        return false;
    KURL::List list;
    
    KURLDrag::decode(event, list);
    KURL url;
    KMimeType::Ptr mime;
    bool canHandle = true;
    for (unsigned int i = 0; i < list.count(); ++i)
    {
        url = list[i];
        mime = KMimeType::findByURL(url);
        if (!mime->is("application/x-desktop"))
        {
            canHandle = false;
            break;
        }
    }
    return canHandle;
}

bool ServicePanel::handleDrop(QDropEvent *event)
{
    if (!KURLDrag::canDecode(event))
        return false;
    KURL::List list;
    
    KURLDrag::decode(event, list);
    KURL url;
    KStandardDirs *dirs = KGlobal::dirs();
    QString location = dirs->saveLocation("appdata", m_name, true);
    QString fileName;
    bool status = true;
    for (unsigned int i = 0; i < list.count(); ++i)
    {
        url = list[i];
        fileName = location + url.fileName();
        if (KIO::NetAccess::copy(url, fileName, this))
        {
            ServiceItem *item = new ServiceItem(this, KService::serviceByStorageId(location + "/" + url.fileName()));
            addItem(item);
        }
        else
        {
            kdDebug() << "kpanels: ERROR: could not copy file: " << url << endl;
            status = false;
        }
        
    }
    return status;
}

#include "servicepanel.moc"
